(* -------------------------------------------------- *)
(* TCPhrase User Interface
(* ѵwJ(TCPhrase) ı(զr)
(*
(*
(* @: F Wolfgang Chien <wolfgang@ms2.hinet.net>
(* -------------------------------------------------- *)
unit cPhsUI;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, Buttons, StdCtrls;

const
  _KeySize = 8;
  _CacheSize = 5 * 1024;
  VK_0 = $30;
  VK_Z = $5A;
  { VK_0 thru VK_9 are the same as ASCII '0' thru '9' ($30 - $39) }
  { VK_A thru VK_Z are the same as ASCII 'A' thru 'Z' ($41 - $5A) }

type
  TfrmUI = class(TForm)
    pnlComp: TPanel;
    Panel2: TPanel;
    btnPhrase: TSpeedButton;
    btnClose: TSpeedButton;
    pnlCompString: TPanel;
    procedure btnCloseClick(Sender: TObject);
    procedure btnPhraseClick(Sender: TObject);
    procedure pnlCompStringMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure pnlCompStringMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    { Private declarations }
    bInMove: Boolean;
    ptLastMousePos: TPoint;
    rtDraw : TRect;
    sCompStr: string;

    hPhraseFile: integer;
    pcCache: pchar;
    iCacheSize: integer;

    procedure WMMouseMove(var msg: TMessage); message WM_MouseMove;
    procedure WMLButtonUP(var msg: TMessage); message WM_LBUTTONUP;
    procedure DrawFocusRect(var R:TRect);
    procedure Final;
  protected
    procedure CreateParams(var Params: TCreateParams); override;
  public
    { Public declarations }
    function IsUIComponent(wParam: THandle): boolean;
    function SearchText(const sKey: string): string;
    procedure SendDBCSString(hFocus: HWND; const sSend: string);
    function Init(const sPhraseFileName: string): boolean;
    function DoKeyEvent(wParam: WPARAM; lParam: LPARAM): boolean;
  end;

var
  frmUI: TfrmUI;
  hFocusControl: HWND;
  bPhsTemp: boolean;
  bPhsKeep: boolean;


implementation

uses CPhrase;

{$R *.DFM}

(* -------------------------------------------------- *)
(*
(*
(*
(* -------------------------------------------------- *)
procedure TfrmUI.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  {
  Params.style := Params.style or ws_Popup or CS_IME;
  Params.ExStyle := 2 * sizeof(LongInt);
  Params.WindowClass.style := Params.WindowClass.style or CS_IME;
  Params.WindowClass.cbWndExtra := 2 * sizeof(LongInt);
  Params.WindowClass.style := Ws_Popup + CS_IME;
  }
  Params.WndParent := Application.MainForm.Handle;
end;

(* -------------------------------------------------- *)
(*
(*
(*
(* -------------------------------------------------- *)
procedure TfrmUI.DrawFocusRect(var R:TRect);
var
  DC : hDC;
  OldBrush:hBrush;
begin
  DC := GetDC(0);
  SetROP2(DC, R2_NOT);
  OldBrush := SelectObject ( DC, GetStockObject ( HOLLOW_BRUSH ) );
  With R Do Rectangle ( DC, Left, Top, Right, Bottom );
  SelectObject( DC, OldBrush );
  ReleaseDC(0, DC);
end;

(* -------------------------------------------------- *)
(*
(*
(*
(* -------------------------------------------------- *)
procedure TfrmUI.pnlCompStringMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if Button = mbLeft then
  begin
    bInMove := True;
    rtDraw := ClientRect;
    MapWindowPoints(Handle, 0, rtDraw, 2);
    DrawFocusRect(rtDraw);
    ptLastMousePos.X := X;
    ptLastMousePos.Y := Y;
    Windows.ClientToScreen(Handle, ptLastMousePos);
    SetCapture(Handle);
  end;
end;


(* -------------------------------------------------- *)
(*
(*
(*
(* -------------------------------------------------- *)
procedure TfrmUI.WMMouseMove(var msg: TMessage);
var
  ptNewMousePos: TPoint;
begin
  if not bInMove then Exit;
  ptNewMousePos := Point(integer((Msg.lParam shl 16) shr 16), integer(Msg.lParam shr 16));
  Windows.ClientToScreen(Handle, ptNewMousePos);
  DrawFocusRect(rtDraw);
  OffsetRect(rtDraw, ptNewMousePos.X - ptLastMousePos.X,
    ptNewMousePos.Y - ptLastMousePos.Y);
  DrawFocusRect(rtDraw);
  ptLastMousePos := ptNewMousePos;

  inherited;
end;

(* -------------------------------------------------- *)
(*
(*
(*
(* -------------------------------------------------- *)
procedure TfrmUI.WMLButtonUP(var msg: TMessage);
var
  ptNewMousePos: TPoint;
begin
  ReleaseCapture;
  bInMove := False;
  DrawFocusRect(rtDraw);
  Left := rtDraw.Left;
  Top := rtDraw.Top;
  Windows.SetFocus(hFocusControl);
  inherited;
end;

(* -------------------------------------------------- *)
(*
(*
(*
(* -------------------------------------------------- *)
procedure TfrmUI.btnCloseClick(Sender: TObject);
begin
  bPhsTemp := False;
  bPhsKeep := False;
  Hide;
end;

(* -------------------------------------------------- *)
(*
(*
(*
(* -------------------------------------------------- *)
function TfrmUI.IsUIComponent(wParam: THandle): boolean;
begin
  if not HandleAllocated then
  begin
    Result := False;
    Exit;
  end;
  Result := True;
  if Handle = wParam then Exit;
  if pnlComp.Handle = wParam then Exit;
  if Panel2.Handle = wParam then Exit;
  if pnlCompString.Handle = wParam then Exit;
  Result := False;
end;

(* -------------------------------------------------- *)
(*
(*
(*
(* -------------------------------------------------- *)
function TfrmUI.SearchText(const sKey: string): string;
var
  pcFound: pchar;
  pcStop: pchar;
  szKey: array[0.._KeySize+1] of char;
  iReadCount: integer;
  szData: array[0..254] of char;
begin
  Result := '';
  if hPhraseFile < 0 then Exit;
  StrPCopy(szKey, sKey);

  pcFound := StrPos(pcCache, szKey);
  if pcFound = nil then
  begin
    FileSeek(hPhraseFile, 0, 0);
    iReadCount := FileRead(hPhraseFile, pcCache^, iCacheSize);
  end
  else
    iReadCount := 1;

  while iReadCount > 0 do
  begin
    if pcFound = nil then
      pcFound := StrPos(pcCache, szKey);
    if pcFound <> nil then
    begin
      Inc(pcFound, Length(sKey));
      pcStop := pcFound;
      repeat
        Inc(pcStop);
      until pcStop^ = #13;
      StrLCopy(szData, pcFound, integer(pcStop - pcFound));
      Result := string(szData);
      Break;
    end;
    iReadCount := FileRead(hPhraseFile, pcCache^, iCacheSize);
  end;
end;

(* -------------------------------------------------- *)
(*
(*
(*
(* -------------------------------------------------- *)
procedure TfrmUI.SendDBCSString(hFocus: HWND; const sSend: string);
var
  hActiveControl: HWND;
  i: integer;
  ch: byte;
begin
  if hFocus = 0 then hFocus := GetFocus;
  if hFocus = 0 then Exit;
  i := 1;
  while i <= Length(sSend) do
  begin
    ch := byte(sSend[i]);
    (* SendMessage(hFocus, WM_CHAR, ch, 0); // , TǤpw *)
    if Windows.IsDBCSLeadByte(ch) then
    begin
      Inc(i);
      SendMessage(hFocus, WM_IME_CHAR, MakeWord(byte(sSend[i]), ch), 0);
    end
    else
      SendMessage(hFocus, WM_IME_CHAR, word(ch), 0);
    Inc(i);
  end;
end;

(* -------------------------------------------------- *)
(*
(*
(*
(* -------------------------------------------------- *)
procedure TfrmUI.btnPhraseClick(Sender: TObject);
var
  sSearchResult: string;
begin
  sSearchResult := SearchText(sCompStr + #10);
  if sSearchResult = '' then
    pnlCompString.Caption := 'dL'
  else
  begin
    SendDBCSString(hFocusControl, sSearchResult);
    pnlCompString.Caption := '';
    if bPhsTemp then
    begin
      bPhsTemp := False;
      Hide;
    end;
  end;
  sCompStr := '';
  Windows.SetFocus(hFocusControl);
end;

(* -------------------------------------------------- *)
(*
(*
(*
(* -------------------------------------------------- *)
function TfrmUI.Init(const sPhraseFileName: string): boolean;
begin
  Final;
  hPhraseFile := FileOpen(sPhraseFileName, fmOpenRead + fmShareDenyNone);
  iCacheSize := FileSeek(hPhraseFile, 0, 2);
  if iCacheSize > _CacheSize then iCacheSize := _CacheSize;
  GetMem(pcCache, iCacheSize + 1);
  FileSeek(hPhraseFile, 0, 0);
  FileRead(hPhraseFile, pcCache^, iCacheSize);
end;

(* -------------------------------------------------- *)
(*
(*
(*
(* -------------------------------------------------- *)
function TfrmUI.DoKeyEvent(wParam: WPARAM; lParam: LPARAM): boolean;
const
  _KeyPressMask = $80000000;
var
  ch: char;
begin
  Result := False;
  // if ((lParam and _KeyPressMask) <> 0) then
  case wParam of
    VK_Escape:
      begin
        sCompStr := '';
        pnlCompString.Caption := '';
        Result := True;
      end;
    VK_BACK:
      begin
        if sCompStr = '' then Exit;
        if (lParam and _KeyPressMask) <> 0 then
        begin
          SetLength(sCompStr, Length(sCompStr) - 1);
          pnlCompString.Caption := sCompStr;
        end;
        Result := True;
      end;
    VK_SPACE:
      begin
        if (lParam and _KeyPressMask) <> 0 then
          if sCompStr <> '' then btnPhraseClick(nil);
        Result := True;
      end;
    VK_0..VK_Z:
      begin
        if (lParam and _KeyPressMask) <> 0 then
        begin
          // ch := char(MapVirtualKey(wParam, 2));
          AppendStr(sCompStr, char(wParam));
          pnlCompString.Caption := sCompStr;
        end;
        Result := True;
      end;
    $C0, $BD, $BB, $DB, $DD, $DC, $BF, $BE, $BC:
      begin
        if (lParam and _KeyPressMask) <> 0 then
        begin
          ch := char(MapVirtualKey(wParam, 2));
          AppendStr(sCompStr, ch);
          pnlCompString.Caption := sCompStr;
        end;
        Result := True;
      end;
  end;
end;

(* -------------------------------------------------- *)
(*
(*
(*
(* -------------------------------------------------- *)
procedure TfrmUI.pnlCompStringMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  Windows.SetFocus(hFocusControl);
end;

(* -------------------------------------------------- *)
(*
(*
(*
(* -------------------------------------------------- *)
procedure TfrmUI.FormCreate(Sender: TObject);
begin
  hPhraseFile := 0;
  iCacheSize := 0;
  Left := Application.MainForm.Left + Application.MainForm.Width - Width;
  Top := Application.MainForm.Top + Application.MainForm.Height - Height;
end;

(* -------------------------------------------------- *)
(*
(*
(*
(* -------------------------------------------------- *)
procedure TfrmUI.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Final;
  Action := caFree;
  frmUI := nil;
end;

(* -------------------------------------------------- *)
(*
(*
(*
(* -------------------------------------------------- *)
procedure TfrmUI.Final;
begin
  if hPhraseFile > 0 then FileClose(hPhraseFile);
  if iCacheSize > 0 then FreeMem(pcCache, iCacheSize + 1);
end;

end.
